@@ -68,6 +68,7 @@ static ValidProperty valid_properties[] = {
 	{ NM_VPNC_KEY_DOMAIN,                G_TYPE_STRING, 0, 0 },
 	{ NM_VPNC_KEY_DHGROUP,               G_TYPE_STRING, 0, 0 },
 	{ NM_VPNC_KEY_PERFECT_FORWARD,       G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_VENDOR,                G_TYPE_STRING, 0, 0 },
 	{ NM_VPNC_KEY_APP_VERSION,           G_TYPE_STRING, 0, 0 },
 	{ NM_VPNC_KEY_SINGLE_DES,            G_TYPE_BOOLEAN, 0, 0 },
 	{ NM_VPNC_KEY_NO_ENCRYPTION,         G_TYPE_BOOLEAN, 0, 0 },
@@ -406,6 +407,9 @@ nm_vpnc_config_write (gint vpnc_fd,
 
 	default_username = nm_setting_vpn_get_user_name (s_vpn);
 
+	if (getenv ("VPNC_DEBUG"))
+		write_config_option (vpnc_fd, "Debug 3\n");
+
 	write_config_option (vpnc_fd, "Script " NM_VPNC_HELPER_PATH "\n");
 
 	write_config_option (vpnc_fd,
@@ -472,6 +476,9 @@ real_connect (NMVPNPlugin   *plugin,
 	if (vpnc_fd < 0)
 		goto out;
 
+	if (getenv ("NM_VPNC_DUMP_CONNECTION"))
+		nm_connection_dump (connection);
+
 	if (!nm_vpnc_config_write (vpnc_fd, s_vpn, error))
 		goto out;
 
@@ -604,9 +611,13 @@ main (int argc, char *argv[])
 
 	main_loop = g_main_loop_new (NULL, FALSE);
 
-	g_signal_connect (plugin, "quit",
-				   G_CALLBACK (quit_mainloop),
-				   main_loop);
+	if (   (argc != 2)
+	    || !argv[1]
+	    || strcmp (argv[1], "--persist")) {
+		g_signal_connect (plugin, "quit",
+					   G_CALLBACK (quit_mainloop),
+					   main_loop);
+	}
 
 	g_main_loop_run (main_loop);
 
